/* MACHINE-GENERATED - DO NOT EDIT ! */

#ifndef _LINUX_ABISS_MSG_H
#define _LINUX_ABISS_MSG_H

/*
 * include/linux/abiss_msg.h - ABISS message definitions
 *
 * Written 2003-2005 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2003-2005. All rights
 * reserved.
 *
 * You can redistribute and/or modify this software under the terms of
 * version 2 of the GNU General Public License as published by the Free
 * Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <stdint.h>
#include <sys/types.h>
#include <linux/types.h>
#include <linux/ioctl.h>


#define ABISS_IOCTL		_IOWR('A',0x80,struct abiss_msg)
#define ABISS_IOCTL_COMPAT	_IOWR('A',0x81,struct abiss_msg)
#define ABISS_NAME_LENGTH	16

#define	ABISS_PRIO_MAX	7


struct abiss_msg {
	enum abiss_msg_type {
		abiss_set,	/* set ABISS on super block */
		abiss_query,	/* query ABISS on super block */
		abiss_attach,	/* attach and enable RT processing */
		abiss_position,	/* set (move) playout point */
		abiss_position_fb, /* set (move) playout point, with feedback */
		abiss_sched_msg,/* scheduler-defined message */
		abiss_stat_set,	/* set up statistics histogram */
		abiss_stat_swap,/* swap counters */
		abiss_stat_get,	/* read counter */
		abiss_direct_mm,/* direct MM control */
	} type;
};


/* ----- abiss_set --------------------------------------------------------- */


struct abiss_set_msg {
	struct abiss_msg header;
	char sched_name[ABISS_NAME_LENGTH];	/* \0 for none */
};


/* ----- abiss_stat_* ------------------------------------------------------ */


/*
 * @@@ Note: all this really ought to be scheduler-specific ...
 */


enum abiss_stat_type {
	abiss_stat_load,	/* time from prefetch to completion of last
				   request */
	abiss_stat_defer_late,	/* delta time for late deferred shift */
	abiss_stat_defer_early,	/* delta time for early deferred shift */
	abiss_stat_behind,	/* distance to playout point, when sliding the
				   window */
	abiss_stat_ahead,	/* distance to playout page, when moving the
				   playout point */
	abiss_stat_slot,	/* slot in which prefetch completes */
	abiss_stat_timer_late,	/* delta time for late timer */
	abiss_stat_timer_early,	/* delta time for early timer */
	abiss_stat_defer_slid,	/* number of slots we slid after deferring */
	abiss_stat_num		/* number of histograms; MUST BE LAST */
};

/*
 * "excess" and "buckets" are only used when returning data from the kernel.
 */

struct abiss_stat_msg {
	struct abiss_msg header;
	enum abiss_stat_type stat; /* histogram number */

	/* for configuration */
	int alloc_buckets;	/* number of counters (not set by query) */
	int shift;		/* number of bits to right-shift; also used in
				   query */

	/* for querying */
	int out_buckets;	/* number of counters in *buckets; in/out */
	int excess;		/* entries beyond end of array */
	int missing;		/* unaccounted-for entries */
	int  *buckets;	/* array of counters; can be NULL */
};


#if defined(__KERNEL__) && defined(CONFIG_COMPAT)

/*
 * Near-clone of "struct abiss_stat_msg", for 32 bit ioctl conversion.
 * Do not use this in applications !
 */

struct abiss_stat_msg_compat {
	struct abiss_msg header;
	enum abiss_stat_type stat;

	int alloc_buckets;
	int shift;

	int out_buckets;
	int excess;
	int missing;
	compat_uptr_t buckets;
};

#endif /* CONFIG_COMPAT */


/* ----- abiss_attach ------------------------------------------------------ */


struct abiss_attach_msg {
	struct abiss_msg header;
	int _pad;				/* align to 8 bytes */
	char sched_name[ABISS_NAME_LENGTH];
	const void  *sched_prm;
};


#if defined(__KERNEL__) && defined(CONFIG_COMPAT)

/*
 * Near-clone of "struct abiss_attach_msg", for 32 bit ioctl conversion.
 * Do not use this in applications !
 */

struct abiss_attach_msg_compat {
	struct abiss_msg header;
	int _pad;				/* align to 8 bytes */
	char sched_name[ABISS_NAME_LENGTH];
	compat_uptr_t sched_prm;
};

#endif /* CONFIG_COMPAT */


/* ----- abiss_position ---------------------------------------------------- */


/*
 * For some obscure reason, SEEK_* are not defined anywhere in the kernel
 * headers.
 */

#ifdef __KERNEL__
#define SEEK_SET	0
#define SEEK_CUR	1
#define SEEK_END	2
#endif /* __KERNEL__ */

enum abiss_seek_mode {
	abiss_seek_rt	= 0,	/* try to stay in RT mode */
	abiss_seek_be	= 1,	/* switch to BE for playout buffer update */
};

struct abiss_position_msg {
	struct abiss_msg header;
	int whence;		/* SEEK_* */
	loff_t pos;		/* new playout point position, in bytes */
	enum abiss_seek_mode seek_mode;
	int _pad;		/* pad to n*8 bytes */
};


/* ----- Power daemon messages --------------------------------------------- */


#define ABISS_PMD_OFF		0	/* power off (from any state) */
#define ABISS_PMD_ON		1	/* power on (from any state) */

#endif /* _LINUX_ABISS_MSG_H */
