/*
 * abissd/close.c - abissd, "close" message
 *
 * Written 2004 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2004. All rights reserved.
 * For licensing and warranty information, see the file ../../COPYING.LGPLv21
 */


#include <stdio.h>

#include <abiss_msg.h>
#include <abissd.h>

#include "common.h"
#include "stream.h"


size_t abissd_close(void *buffer)
{
    struct abissd_close_msg *msg = buffer;
    struct stream *stream;

    if (verbose > 1) {
	fprintf(stderr,"close id: dev 0x%llx, ino %llu, fid 0x%llx\n",
	    (unsigned long long) msg->id.dev,
	    (unsigned long long) msg->id.ino,
	    (unsigned long long) msg->id.fid);
    }

    stream = find_stream(&msg->id);
    if (!stream) {
	fprintf(stderr,"cannot close file (not in list)\n");
	return sizeof(*msg);
    }
  
    if (stream->sched_ops->close)
	stream->sched_ops->close(stream);
    destroy_stream(stream);

    return sizeof(*msg);
}
