/*
 * seq.c - Sequence number access
 *
 * Written 2005 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2005. All rights reserved.
 * For licensing and warranty information, see the file ../../COPYING.GPLv2
 */

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <assert.h>

#include "seq.h"


uint32_t seq;

static int next_seq = 0;
static int curr_edge = 0;


void reset_sequence(void)
{
    next_seq = 0;
    curr_edge = 0;
}


static int do_sequence(void *buffer,int size,
  int (*fn)(void *seq_frag,void *buf_frag,int size))
{
    int error = 0;

    while (next_seq < curr_edge+size) {
	int into,offset,left,len,res;

	assert(next_seq+4 > curr_edge);
	if (next_seq >= curr_edge) {
	    into = 0;
	    offset = next_seq-curr_edge;
	}
	else {
	    into = curr_edge-next_seq;
	    offset = 0;
	}
	left = size-offset;
	len = left < 4-into ? left : 4-into;
	res = fn((void *) &seq+into,buffer+offset,len);
	if (res && !error)
	    error = res;
	if (len+into < 4)
	    break;
	next_seq += 1024;
	seq++;
    }
    curr_edge += size;
    return error;
}


static int do_put(void *seq_frag,void *buf_frag,int size)
{
    memcpy(buf_frag,seq_frag,size);
    return 0;
}


void put_sequence(void *buffer,int size)
{
    do_sequence(buffer,size,do_put);
}


static int do_check(void *seq_frag,void *buf_frag,int size)
{
    int res;

    res = memcmp(buf_frag,seq_frag,size);
    if (res)
	memcpy(seq_frag,buf_frag,size);
    return res;
}


int check_sequence(void *buffer,int size)
{
    return do_sequence(buffer,size,do_check);
}
