/*
 * Common.h - C framework for RB tree unit tests
 *
 * Written 2003-2004 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2003-2004. All rights
 * reserved.
 *
 * For licensing and warranty information, see the file ../../COPYING.GPLv2
 */


#include <stdio.h>
#include <string.h>
#include <assert.h>


struct rb_node {
    struct rb_node *rb_parent;
    struct rb_node *rb_right;
    struct rb_node *rb_left;
    int key;
};

static struct rb_root {
    struct rb_node *rb_node;
} root;


#define BUG_ON(x) assert(!x)

#include RBWALK_H


static void __attribute__((used)) __add(struct rb_node *node)
{
    struct rb_node **n;
    struct rb_node *parent = NULL;

    n = &root.rb_node;
    while (*n) {
	parent = *n;
	if (node->key < (*n)->key) n = &(*n)->rb_left;
	else n = &(*n)->rb_right;
    }
    *n = node;
    node->rb_parent = parent;
    node->rb_left = node->rb_right = NULL;
}


#define add(k) \
    { static struct rb_node node = { .key = k }; \
      __add(&node); }

#define tree0	/* do nothing */

#define tree1 \
    add(1);

#define treeN \
    add(5); \
    add(3); \
    add(2); \
    add(1); \
    add(4); \
    add(8); \
    add(6); \
    add(7); \


static void visit(int at_end)
{
    struct rb_node *walk;
    enum rb_walk_state state;

    __rb_walk_start(&root,&walk,&state);
    while (1) {
	struct rb_node *n;

	n = __rb_walk(&walk,&state,at_end);
	if (!n)
		return;
	printf("%d\n",n->key);
	if (at_end) memset(n,0,sizeof(*n));
    }
}
